# A simple generator wrapper from https://gist.github.com/everilae/9697228
# With basic python threading
from queue import Queue
from threading import Thread


class ThreadedGenerator:
    """
    Generator that runs on a separate thread, returning values to calling
    thread. Care must be taken that the iterator does not mutate any shared
    variables referenced in the calling thread.
    """

    def __init__(self, iterator,
                 sentinel=object(),
                 queue_maxsize=0,
                 daemon=False):
        self._iterator = iterator
        self._sentinel = sentinel
        self._queue = Queue(maxsize=queue_maxsize)
        self._thread = Thread(
            name=repr(iterator),
            target=self._run
        )
        self._thread.daemon = daemon

    def __repr__(self):
        return 'ThreadedGenerator({!r})'.format(self._iterator)

    def _run(self):
        try:
            for value in self._iterator:
                self._queue.put(value)

        finally:
            self._queue.put(self._sentinel)

    def __iter__(self):
        self._thread.start()
        for value in iter(self._queue.get, self._sentinel):
            yield value

        self._thread.join()



import os
from configargparse import ArgParser

from tupa.model import NODE_LABEL_KEY
from tupa.scripts.export import load_model


def strip_multitask(model, keep):
    if "amr" not in keep:
        for d in (model.feature_params, model.classifier.params):
            for key in (NODE_LABEL_KEY, "c"):
                try:
                    del d[key]
                except KeyError:
                    pass
    for d in (model.classifier.labels, model.classifier.axes):
        for axis in set(d).difference(keep):
            try:
                del d[axis]
            except KeyError:
                pass


def main(args):
    os.makedirs(args.out_dir, exist_ok=True)
    for filename in args.models:
        model = load_model(filename)
        strip_multitask(model, args.keep)
        model.filename = os.path.join(args.out_dir, os.path.basename(filename))
        model.save()


if __name__ == "__main__":
    argparser = ArgParser(description="Load TUPA model and save with just one task's features/weights.")
    argparser.add_argument("models", nargs="+", help="model file basename(s) to load")
    argparser.add_argument("-k", "--keep", nargs="+", default=["ucca"], help="tasks to keep features/weights for")
    argparser.add_argument("-o", "--out-dir", default=".", help="filename suffix to append")
    main(argparser.parse_args())



import matplotlib
import numpy as np

matplotlib.use("Agg")
import matplotlib.pyplot as plt
from configargparse import ArgParser

np.seterr("raise")

PRIMARY_F1_COLUMN = 3
REMOTE_F1_COLUMN = 6


def load_scores(filename):
    print("Loading dev scores from '%s'" % filename)
    scores = np.loadtxt(filename + ".dev.csv", delimiter=",", skiprows=1)
    try:
        return scores[:, [PRIMARY_F1_COLUMN, REMOTE_F1_COLUMN]]
    except IndexError:
        return scores[:, PRIMARY_F1_COLUMN]


def visualize(scores, filename):
    plt.plot(range(1, 1 + len(scores)), scores)
    plt.xlabel("epochs")
    plt.ylabel("dev f1")
    plt.legend(["primary", "remote"])
    plt.title(filename)
    output_file = filename + ".dev.png"
    plt.savefig(output_file)
    plt.clf()
    print("Saved '%s'." % output_file)


def main():
    argparser = ArgParser(description="Visualize scores of a model over the dev set, saving to .png file.")
    argparser.add_argument("models", nargs="+", help="model file basename(s) to load")
    args = argparser.parse_args()
    for filename in args.models:
        scores = load_scores(filename)
        visualize(scores, filename)


if __name__ == "__main__":
    main()
